import json
import os
import itertools
import more_itertools as mit
from collections import defaultdict

import regex as re
import logging

from datetime import datetime
from .config import ASSET_PATH, LOG_PATH, TRANSLATION_TABLE

# Should store these separately, but how to deal with escape characters?

period_btw_letters = r'(?<=\w)\.(?=\w)'
period_beginning_or_ending_word = r'(?<=\s)\.|\.(?=\s)'

plus_btw_letters = r'(?<=\w)\+(?=\w)'
plus_beginning_or_ending_word = r'(?<=\s)\+|\+(?=\s)'

surrounding_punctuation = r'(?<=\w)[\.,\+\\\/\(\)]|[\.,\+\\\/\(\)](?=\w)'

letter = r'\b(\w+)\b'
delimiter = r'[\.,\+\\\/\(\)]'
two_letter_abbrev = r'\b' + letter + delimiter + letter
three_letter_abbrev = r'\b' + letter + delimiter + letter + delimiter + letter
pattern = r'(\w+(?:\s+\w+)*)(?:\b[' + delimiter + r']\b|$)'

class Tokenizer:
    def __init__(self, aStr):

        self.logger = logging.getLogger(__name__)
        logging.basicConfig(filemode='w', level=logging.DEBUG, filename=os.path.join(LOG_PATH,f'{datetime.now().strftime("%m-%d-%Y")}.log'), format = '%(asctime)s: %(message)s')

        self.ttable = json.load(open(os.path.join(ASSET_PATH, TRANSLATION_TABLE)))
        try:
            self.aStr = aStr.lower()
        except AttributeError:
            msg = f'Input was {aStr}. Must be a string.'
            self.logger.error(msg)
            self.aStr = ""

        # Flatten translation table for performance.
        self.ttable_dict = {item['original']: item['standardized']
                            for item in self.ttable}

        self.json_repr = defaultdict(list)
        self.standardized, self.cleaned, self.unigrams = self.standardize()
        self.tokens = self.tokenize()


    def __repr__(self):
        pass

    def standardize(self):
        cleaned = re.sub(period_btw_letters, ' ', self.aStr)
        cleaned = re.sub(plus_btw_letters, ' ', cleaned)
        cleaned = re.sub(period_beginning_or_ending_word, '', cleaned)
        cleaned = re.sub(plus_beginning_or_ending_word, '', cleaned)
        cleaned = re.sub(surrounding_punctuation, '', cleaned)

        # Standardize unigrams
        unigrams = cleaned.split()
        unstandardized = unigrams.copy()
        while len(unstandardized) > 0:
            print(f'Unstandardized: {unstandardized}')
            term = unstandardized.pop(0)
            print(f'Popped {term} from {unstandardized}')
            print(f'Checking {term} in self.ttable_dict: {term in self.ttable_dict}')
            if term in self.ttable_dict and term != self.ttable_dict[term]:
                # 2nd boolean term is a hack to prevent infinite loops. 
                # TODO: edit ttable so no entry has a key that is also its value.
                print(f'Replacing {term} with {self.ttable_dict[term]}')
                unigrams[unigrams.index(term)] = self.ttable_dict[term].strip()
                self.logger.info(f'Converted {term} to {self.ttable_dict[term]}')
                self.json_repr['unigrams'].append({'original':term, 'standardized': self.ttable_dict[term]})
                if self.ttable_dict[term] in self.ttable_dict:
                    print(f'Checking {self.ttable_dict[term]} in self.ttable_dict: {self.ttable_dict[term] in self.ttable_dict}')
                    unstandardized.insert(0, self.ttable_dict[term])

        # Standardize bigrams
        standardized = ' '.join(unigrams)
        for bigram in mit.windowed(unigrams, n=2, step=1):
            bigram = ' '.join([word for word in bigram if word is not None])
            if len(bigram.split())==2 and bigram in self.ttable_dict:
                standardized.replace(bigram, self.ttable_dict[bigram])
                self.json_repr['bigrams'].append({'original': bigram, 'standardized': self.ttable_dict[bigram]})
        if self.aStr != standardized:
            self.logger.info(f'Converted \n\t {self.aStr} to \n\t{standardized}')
        else:
            self.logger.info(f'No conversion performed for {self.aStr}')
        return (standardized, cleaned, unigrams)

    def tokenize(self):
        matches = re.findall(pattern, self.aStr)
        return list(matches)

    def to_json(self):
        return json.dumps({'original': self.aStr,
                           'standardized': self.standardized,
                           'tokens': self.json_repr})

__all__ = ['Tokenizer']



import os

ASSET_PATH = os.path.join('assets')
LOG_PATH = os.path.join('logs')
TRANSLATION_TABLE = 'translation_table.clinical_diagnosis.final.json'

for path in [ASSET_PATH, LOG_PATH]:
    if not os.path.exists(path):
        os.makedirs(path)




class Evaluator:
    def __init__(self, filename=None, data=None):
        self.filename = filename
        self.data = data

        if filename:
            pass
        elif data and not filename:
            pass
        elif data and filename:
            pass
        else:
            raise Exception('No data provided.')


    def evaluate_single(self, Toker, gold_standard=None):
        print('Evaluating single...')

        original = Toker.aStr
        processed = Toker.standardized

        print(f'Original: {original}')
        print(f'Processed: {processed}')

        if gold_standard:
            print(f'Gold Standard: {gold_standard}')

    def evaluate(self):
        pass
        # data format: [original, standardized, gold_standard]




import pandas as pd
import json
import os

from tqdm import tqdm
from config import ASSET_PATH
from Tokenizer import Tokenizer as Toker
from Evaluator import Evaluator as Evalor

# To gauge bulk performance
tqdm.pandas()

data = pd.read_excel(os.path.join(ASSET_PATH,'2nd round of iteration.xlsx'))
data = data[['source', 'original', 'standardized', 'change', 'needed', 'flag for review', 'notes']]

sample = data['original'].sample(10)

# Tokenize sample rows.
toker_list = [Toker(row) for row in sample]
for token in toker_list:
    print(f'Original String: {token.aStr}')
    print(f'Standardized String: {token.standardized}')

# # Bulk Conversion
# xf = data.copy(deep=True)
# xf['standardized'] = xf['original'].progress_apply(lambda x: Toker(x).standardized)

# Calculate Performance Metrics
# evil = Evalor()
#
# for toker in toker_list:
#     evil.evaluate_single(toker) # Default summarizes with no comparison to gold standard.

# Explain Processing

# Save to JSONL
SAVE_PATH = os.path.join(ASSET_PATH)
filename = 'test'
save_name = os.path.join(SAVE_PATH, f'{filename}.jsonl')
for toker in toker_list:
    with open(save_name, 'a') as f:
        f.write(toker.to_json())
        f.write('\n')



from setuptools import setup, find_packages

VERSION = '0.1.28' #apparently ignored 
DESCRIPTION = 'Text Regularization for Unstructured Text from EMS Reports'
LONG_DESCRIPTION = 'Made by Michael Chary'

# Setting up
setup(
    name="hospitalpy",
    version=VERSION,
    author="Michael Chary",
    author_email="<mic9189@med.cornell.edu>",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(where='src'),
    include_package_data=True,
    keywords=['python', 'nlp', 'ems', 'global ems'],
    classifiers=[]
)
